#version 330 core

#define EDGE_HORVERT 16
#define EDGE_DIAGONAL_ULLR 32
#define EDGE_DIAGONAL_LLUR 64

//Neighborhood Flags
#define HAS_NORTHERN_NEIGHBOR 1
#define HAS_EASTERN_NEIGHBOR 2
#define HAS_SOUTHERN_NEIGHBOR 4
#define HAS_WESTERN_NEIGHBOR 8
#define HAS_NORTHERN_SPLINE 16
#define HAS_EASTERN_SPLINE 32
#define HAS_SOUTHERN_SPLINE 64
#define HAS_WESTERN_SPLINE 128
#define HAS_CORRECTED_POSITION 256

//Similaritygraph per-node Directions 
#define NORTH		128
#define NORTHEAST	64
#define EAST		32
#define SOUTHEAST	16
#define SOUTH		8
#define SOUTHWEST	4
#define WEST		2
#define NORTHWEST	1

//Constants
#define STEP 0.2
#define GAUSS_MULTIPLIER 3.0
#define LINEAR_SEGMENT_LENGTH 1.0

layout(pixel_center_integer) in vec4 gl_FragCoord;
layout(location = 0) out vec4 color0;

uniform sampler2D pixelArt;
uniform sampler2D similarityGraph;
uniform samplerBuffer indexedCellPositions;
uniform isamplerBuffer neighborIndices;
uniform isamplerBuffer CPflags;
uniform ivec2 viewportDimensions;

//----------------------------------------------------------------
// simple method for fetching cell-graph neighbor indices
//----------------------------------------------------------------
int getNeighborIndex(int sourceIndex, int dir, int targetSector) {
	int index = -1;

	if( (dir & NORTH) == NORTH) {
		index = texelFetch(neighborIndices,sourceIndex).x;
	} else if( (dir & EAST) == EAST) {
		index = texelFetch(neighborIndices,sourceIndex).y;
	} else if( (dir & SOUTH) == SOUTH) {
		index = texelFetch(neighborIndices,sourceIndex).z;
	} else if( (dir & WEST) == WEST) {
		index = texelFetch(neighborIndices,sourceIndex).w;
	}
	return index;
}
//----------------------------------------------------------------

//----------------------------------------------------------------
// calulates the spline point at parametric position t
//----------------------------------------------------------------
vec2 calcSplinePoint(vec2 p0, vec2 p1, vec2 p2, float t) {
	float t2 = 0.5*t*t;
	float a = t2 - t + 0.5;
	float b = -2.0 * t2 + t + 0.5;
	return a*p0 + b*p1 + t2*p2;
}
//----------------------------------------------------------------

//----------------------------------------------------------------
// calulates the spline endpoint at parametric position t
//----------------------------------------------------------------
vec2 calcSplineEndPoint(vec2 p0, vec2 p1, float t) {
	float t2 = 0.5*t*t;
	return (-t2+1)*p0 + t2*p1;
}
//----------------------------------------------------------------

//----------------------------------------------------------------
// test two lines for intersection
// source: http://stackoverflow.com/questions/563198/how-do-you-detect-where-two-line-segments-intersect
//----------------------------------------------------------------
bool interects(vec2 lineApointA, vec2 lineApointB, vec2 lineBpointA, vec2 lineBpointB) {
	vec2 r = lineApointB - lineApointA;
	vec2 s = lineBpointB - lineBpointA;
	float rXs = r.x * s.y - r.y * s.x;
	if (rXs == 0.0) {
		return false;
	}
	vec2 ba = lineBpointA - lineApointA;
	float t = (ba.x * s.y - ba.y * s.x)/rXs;
	if( (t < 0.0) || (t > 1.0) ) {
		return false;
	}
	float u = (ba.x * r.y - ba.y * r.x)/rXs;
	if( (u < 0.0) || (u > 1.0) ) {
		return false;
	}
	return true;
}
//----------------------------------------------------------------

void main() {
	// convert fragment-space coordinates to cell-space coordinates
	vec2 cellSpaceCoords = (textureSize(pixelArt,0) - 1) / vec2(viewportDimensions - 1) * gl_FragCoord.xy;
	
	// Knot Buffer lookup Coordinates
	int baseKnotIndex = int (2 * floor(cellSpaceCoords.x) + floor(cellSpaceCoords.y) * 2 * (textureSize(pixelArt,0).x - 1) );
	
	// create a boolean vector containing information on the influence of surrounding pixels  on this fragment 
	// R G B A ... UL UR LL LR
	bvec4 influencingPixels = bvec4(true,true,true,true);
	
	// surrounding pixel Coordinates 
	vec2 ULCoords = vec2(floor(cellSpaceCoords.x),ceil(cellSpaceCoords.y));
	vec2 URCoords = vec2(ceil(cellSpaceCoords.x),ceil(cellSpaceCoords.y));
	vec2 LLCoords = vec2(floor(cellSpaceCoords.x),floor(cellSpaceCoords.y));
	vec2 LRCoords = vec2(ceil(cellSpaceCoords.x),floor(cellSpaceCoords.y));
	
	bool hasCorrectedPosition = false;
	
	vec2 knot0pos = texelFetch(indexedCellPositions,baseKnotIndex).rg;
	
	int flags0 = texelFetch(CPflags,baseKnotIndex).r;
	if(flags0 >= 0.0) {
		
		vec2 correctedPosition = knot0pos;
		// check if this point lies on a T-Junction and save its corrected Position
		if ( (flags0 & HAS_CORRECTED_POSITION) == HAS_CORRECTED_POSITION ) {
			hasCorrectedPosition = true;
			correctedPosition = texelFetch(indexedCellPositions,baseKnotIndex + 1).rg;
		}
	
		//gather neighbors
		ivec4 neighbors = texelFetch(neighborIndices,baseKnotIndex);
		
		
		vec2 linearNeighbors[4];
		int linearCount = 0;
		/*
		//==================================INSERT_BEGIN=============================================
		vec2 linearNeighborsNeighbors[4] = vec2[4](vec2(0.0,0.0),vec2(0.0,0.0),vec2(0.0,0.0),vec2(0.0,0.0)); //ADD
		bool linearneighborHasNeighbor[4] = bool[4](false,false,false,false); //ADD
		//====================================INSERT_END=============================================
		*/
		vec2 splineNeighbors[2];
		int splineCount = 0;
		
		vec2 splineNeighborsNeighbors[2] = vec2[2](vec2(0.0,0.0),vec2(0.0,0.0));
		bool neighborHasNeighbor[2] = bool[2](false,false);
		
		if( (flags0 & HAS_NORTHERN_NEIGHBOR) == HAS_NORTHERN_NEIGHBOR ) {
			int neighborflags = texelFetch(CPflags,neighbors.x).r;
			if( (flags0 & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE ) {
				if(neighborflags >= HAS_NORTHERN_SPLINE && !( (neighborflags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE )) { // hmm ... why did i do this .. TODO check if HAS SOUTHERN SPLINE would be sufficient
					splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.x+1).rg;
				} else {
					splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.x).rg;
					//investigate neighbors neighborhood and add control points if there are any
					if( (neighborflags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE) {
						//get neighbors neighbors flag
						int neighborsNeighborIndex = getNeighborIndex(neighbors.x, NORTH, 0);
						int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
						if ((neighborsNeighborflags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE) {
							//take this one
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
							neighborHasNeighbor[splineCount]=true;
						} else {
							// use the corrected position
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
							neighborHasNeighbor[splineCount]=true;
						}
					}
						//investigate neighbors neighborhood and add control points if there are any
					if( (neighborflags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE) {
						//get neighbors neighbors flag
						int neighborsNeighborIndex = getNeighborIndex(neighbors.x, EAST, 0);
						int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
						if ((neighborsNeighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE) {
							//take this one
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
							neighborHasNeighbor[splineCount]=true;
						} else {
							// use the corrected position
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
							neighborHasNeighbor[splineCount]=true;
						}
					}
					if( (neighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE) {
						//get neighbors neighbors flag
						int neighborsNeighborIndex = getNeighborIndex(neighbors.x, WEST, 0);
						int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
						if ((neighborsNeighborflags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE) {
							//take this one
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
							neighborHasNeighbor[splineCount]=true;
						} else {
							// use the corrected position
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
							neighborHasNeighbor[splineCount]=true;
						}
					}
				}
				splineCount++;
			} else if ( (neighborflags & HAS_CORRECTED_POSITION) == HAS_CORRECTED_POSITION) {
				linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.x+1).rg;
				linearCount++;
			} else {
				linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.x).rg;
				linearCount++;
			}
		}
		
		if( (flags0 & HAS_EASTERN_NEIGHBOR) == HAS_EASTERN_NEIGHBOR ) {
			int neighborflags = texelFetch(CPflags,neighbors.y).r;
			if( (flags0 & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE ) {
				if(neighborflags >= HAS_NORTHERN_SPLINE && !( (neighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE )) {
					splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.y+1).rg;
				} else {
					splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.y).rg;
					//investigate neighbors neighborhood and add control points if there are any
					if( (neighborflags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE) {
						//get neighbors neighbors flag
						int neighborsNeighborIndex = getNeighborIndex(neighbors.y, NORTH, 0);
						int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
						if ((neighborsNeighborflags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE) {
							//take this one
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
							neighborHasNeighbor[splineCount]=true;
						} else {
							// use the corrected position
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
							neighborHasNeighbor[splineCount]=true;
						}
					}
						//investigate neighbors neighborhood and add control points if there are any
					if( (neighborflags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE) {
						//get neighbors neighbors flag
						int neighborsNeighborIndex = getNeighborIndex(neighbors.y, EAST, 0);
						int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
						if ((neighborsNeighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE) {
							//take this one
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
							neighborHasNeighbor[splineCount]=true;
						} else {
							// use the corrected position
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
							neighborHasNeighbor[splineCount]=true;
						}
					}
					if( (neighborflags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE) {
						//get neighbors neighbors flag
						int neighborsNeighborIndex = getNeighborIndex(neighbors.y, SOUTH, 0);
						int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
						if ((neighborsNeighborflags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE) {
							//take this one
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
							neighborHasNeighbor[splineCount]=true;
						} else {
							// use the corrected position
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
							neighborHasNeighbor[splineCount]=true;
						}
					}
				}
				splineCount++;
			} else if ( (neighborflags & HAS_CORRECTED_POSITION) == HAS_CORRECTED_POSITION) {
				linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.y+1).rg;
				linearCount++;
			} else {
				linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.y).rg;
				linearCount++;
			}
		}
		
		if( (flags0 & HAS_SOUTHERN_NEIGHBOR) == HAS_SOUTHERN_NEIGHBOR ) {
			int neighborflags = texelFetch(CPflags,neighbors.z).r;
			if( (flags0 & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE ) {
				if(neighborflags >= HAS_NORTHERN_SPLINE && !( (neighborflags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE )) {
					splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.z+1).rg;
				} else {
					splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.z).rg;
					//investigate neighbors neighborhood and add control points if there are any
					if( (neighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE) {
						//get neighbors neighbors flag
						int neighborsNeighborIndex = getNeighborIndex(neighbors.z, WEST, 0);
						int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
						if ((neighborsNeighborflags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE) {
							//take this one
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
							neighborHasNeighbor[splineCount]=true;
						} else {
							// use the corrected position
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
							neighborHasNeighbor[splineCount]=true;
						}
					}
					if( (neighborflags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE) {
						//get neighbors neighbors flag
						int neighborsNeighborIndex = getNeighborIndex(neighbors.z, EAST, 0);
						int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
						if ((neighborsNeighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE) {
							//take this one
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
							neighborHasNeighbor[splineCount]=true;
						} else {
							// use the corrected position
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
							neighborHasNeighbor[splineCount]=true;
						}
					}
					if( (neighborflags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE) {
						//get neighbors neighbors flag
						int neighborsNeighborIndex = getNeighborIndex(neighbors.z, SOUTH, 0);
						int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
						if ((neighborsNeighborflags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE) {
							//take this one
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
							neighborHasNeighbor[splineCount]=true;
						} else {
							// use the corrected position
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
							neighborHasNeighbor[splineCount]=true;
						}
					}
				}
				splineCount++;
			} else if ( (neighborflags & HAS_CORRECTED_POSITION) == HAS_CORRECTED_POSITION) {
				linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.z+1).rg;
				linearCount++;
			} else {
				linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.z).rg;
				linearCount++;
			}
		}
		
		if( (flags0 & HAS_WESTERN_NEIGHBOR) == HAS_WESTERN_NEIGHBOR ) {
			int neighborflags = texelFetch(CPflags,neighbors.w).r;
			if( (flags0 & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE ) {
				if(neighborflags >= HAS_NORTHERN_SPLINE && !( (neighborflags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE )) {
					splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.w+1).rg;
				} else {
					splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.w).rg;
					//investigate neighbors neighborhood and add control points if there are any
					if( (neighborflags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE) {
						//get neighbors neighbors flag
						int neighborsNeighborIndex = getNeighborIndex(neighbors.w, NORTH, 0);
						int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
						if ((neighborsNeighborflags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE) {
							//take this one
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
							neighborHasNeighbor[splineCount]=true;
						} else {
							// use the corrected position
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
							neighborHasNeighbor[splineCount]=true;
						}
					}
						//investigate neighbors neighborhood and add control points if there are any
					if( (neighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE) {
						//get neighbors neighbors flag
						int neighborsNeighborIndex = getNeighborIndex(neighbors.w, WEST, 0);
						int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
						if ((neighborsNeighborflags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE) {
							//take this one
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
							neighborHasNeighbor[splineCount]=true;
						} else {
							// use the corrected position
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
							neighborHasNeighbor[splineCount]=true;
						}
					}
					if( (neighborflags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE) {
						//get neighbors neighbors flag
						int neighborsNeighborIndex = getNeighborIndex(neighbors.w, SOUTH, 0);
						int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
						if ((neighborsNeighborflags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE) {
							//take this one
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
							neighborHasNeighbor[splineCount]=true;
						} else {
							// use the corrected position
							splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
							neighborHasNeighbor[splineCount]=true;
						}
					}
				}
				splineCount++;
			} else if ( (neighborflags & HAS_CORRECTED_POSITION) == HAS_CORRECTED_POSITION) {
				linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.w+1).rg;
				linearCount++;
			} else {
				linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.w).rg;
				linearCount++;
			}
		}
		//find intersections to determine pixel influence
		if(splineCount == 2) {
			if(neighborHasNeighbor[0]) {
				vec2 pA = calcSplinePoint(splineNeighborsNeighbors[0], splineNeighbors[0], knot0pos, 0.0);
				for (float t = STEP ; t <= 1.0; t = t + STEP) {
					vec2 pointB = calcSplinePoint(splineNeighborsNeighbors[0], splineNeighbors[0], knot0pos, t);
					//evaluate interections
					if( interects(cellSpaceCoords, ULCoords, pA, pointB) ) {
						influencingPixels.x = false;
					}
					if( interects(cellSpaceCoords, URCoords, pA, pointB) ) {
						influencingPixels.y = false;
					}
					if( interects(cellSpaceCoords, LLCoords, pA, pointB) ) {
						influencingPixels.z = false;
					}
					if( interects(cellSpaceCoords, LRCoords, pA, pointB) ) {
						influencingPixels.w = false;
					}
					pA = pointB;
				}
			}
			vec2 pointA = calcSplinePoint(splineNeighbors[0], knot0pos, splineNeighbors[1], 0.0);
			for (float t = STEP ; t <= 1.0; t = t + STEP) {
				vec2 pointB = calcSplinePoint(splineNeighbors[0], knot0pos, splineNeighbors[1], t);
				//evaluate interections
				if( interects(cellSpaceCoords, ULCoords, pointA, pointB) ) {
					influencingPixels.x = false;
				}
				if( interects(cellSpaceCoords, URCoords, pointA, pointB) ) {
					influencingPixels.y = false;
				}
				if( interects(cellSpaceCoords, LLCoords, pointA, pointB) ) {
					influencingPixels.z = false;
				}
				if( interects(cellSpaceCoords, LRCoords, pointA, pointB) ) {
					influencingPixels.w = false;
				}
				pointA = pointB;
			}
			if(neighborHasNeighbor[1]) {
				vec2 pA = calcSplinePoint(knot0pos, splineNeighbors[1], splineNeighborsNeighbors[1], 0.0);
				for (float t = STEP ; t <= 1.0; t = t + STEP) {
					vec2 pointB = calcSplinePoint(knot0pos, splineNeighbors[1], splineNeighborsNeighbors[1], t);
					//evaluate interections
					if( interects(cellSpaceCoords, ULCoords, pA, pointB) ) {
						influencingPixels.x = false;
					}
					if( interects(cellSpaceCoords, URCoords, pA, pointB) ) {
						influencingPixels.y = false;
					}
					if( interects(cellSpaceCoords, LLCoords, pA, pointB) ) {
						influencingPixels.z = false;
					}
					if( interects(cellSpaceCoords, LRCoords, pA, pointB) ) {
						influencingPixels.w = false;
					}
					pA = pointB;
				}
			}
		}
		for(int i = 0 ; i < linearCount ; i++) {
			vec2 pointA = correctedPosition;
			/*
			for (float t = STEP ; t <= 1.0; t = t + STEP + LINEAR_SEGMENT_LENGTH) {
				vec2 pointB = calcSplineEndPoint(correctedPosition, linearNeighbors[i], t);
				//evaluate interections
				if( interects(cellSpaceCoords, ULCoords, pA, pointB) ) {
					influencingPixels.x = false;
				}
				if( interects(cellSpaceCoords, URCoords, pA, pointB) ) {
					influencingPixels.y = false;
				}
				if( interects(cellSpaceCoords, LLCoords, pA, pointB) ) {
					influencingPixels.z = false;
				}
				if( interects(cellSpaceCoords, LRCoords, pA, pointB) ) {
					influencingPixels.w = false;
				}
				pA = pointB;
			}
			*/
			vec2 pointB = mix(correctedPosition, linearNeighbors[i], LINEAR_SEGMENT_LENGTH);
			if( interects(cellSpaceCoords, ULCoords, pointA, pointB) ) {
				influencingPixels.x = false;
			}
			if( interects(cellSpaceCoords, URCoords, pointA, pointB) ) {
				influencingPixels.y = false;
			}
			if( interects(cellSpaceCoords, LLCoords, pointA, pointB) ) {
				influencingPixels.z = false;
			}
			if( interects(cellSpaceCoords, LRCoords, pointA, pointB) ) {
				influencingPixels.w = false;
			}
		}
	}
	//investigate second Knot
	if(!hasCorrectedPosition) {
		vec2 knot1pos = texelFetch(indexedCellPositions,baseKnotIndex + 1).rg;
		int flags = texelFetch(CPflags,baseKnotIndex + 1).r;
		if(flags >= 0.0) {
			
			
			//gather neighbors
			ivec4 neighbors = texelFetch(neighborIndices,baseKnotIndex + 1);
			
			vec2 linearNeighbors[4];
			int linearCount = 0;

			vec2 splineNeighbors[2];
			int splineCount = 0;
			
			vec2 splineNeighborsNeighbors[2] = vec2[2](vec2(0.0,0.0),vec2(0.0,0.0));
			bool neighborHasNeighbor[2] = bool[2](false,false);
			
			if( (flags & HAS_NORTHERN_NEIGHBOR) == HAS_NORTHERN_NEIGHBOR ) {
				int neighborflags = texelFetch(CPflags,neighbors.x).r;
				if( (flags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE ) {
					if(neighborflags >= HAS_NORTHERN_SPLINE && !( (neighborflags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE )) { // hmm ... why did i do this .. TODO check if HAS SOUTHERN SPLINE would be sufficient
						splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.x+1).rg;
					} else {
						splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.x).rg;
						//investigate neighbors neighborhood and add control points if there are any
						if( (neighborflags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE) {
							//get neighbors neighbors flag
							int neighborsNeighborIndex = getNeighborIndex(neighbors.x, NORTH, 0);
							int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
							if ((neighborsNeighborflags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE) {
								//take this one
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
								neighborHasNeighbor[splineCount]=true;
							} else {
								// use the corrected position
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
								neighborHasNeighbor[splineCount]=true;
							}
						}
							//investigate neighbors neighborhood and add control points if there are any
						if( (neighborflags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE) {
							//get neighbors neighbors flag
							int neighborsNeighborIndex = getNeighborIndex(neighbors.x, EAST, 0);
							int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
							if ((neighborsNeighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE) {
								//take this one
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
								neighborHasNeighbor[splineCount]=true;
							} else {
								// use the corrected position
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
								neighborHasNeighbor[splineCount]=true;
							}
						}
						if( (neighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE) {
							//get neighbors neighbors flag
							int neighborsNeighborIndex = getNeighborIndex(neighbors.x, WEST, 0);
							int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
							if ((neighborsNeighborflags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE) {
								//take this one
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
								neighborHasNeighbor[splineCount]=true;
							} else {
								// use the corrected position
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
								neighborHasNeighbor[splineCount]=true;
							}
						}
					}
					splineCount++;
				} else if ( (neighborflags & HAS_CORRECTED_POSITION) == HAS_CORRECTED_POSITION) {
					linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.x+1).rg;
					linearCount++;
				} else {
					linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.x).rg;
					linearCount++;
				}
			}
			
			if( (flags & HAS_EASTERN_NEIGHBOR) == HAS_EASTERN_NEIGHBOR ) {
				int neighborflags = texelFetch(CPflags,neighbors.y).r;
				if( (flags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE ) {
					if(neighborflags >= HAS_NORTHERN_SPLINE && !( (neighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE )) {
						splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.y+1).rg;
					} else {
						splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.y).rg;
						//investigate neighbors neighborhood and add control points if there are any
						if( (neighborflags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE) {
							//get neighbors neighbors flag
							int neighborsNeighborIndex = getNeighborIndex(neighbors.y, NORTH, 0);
							int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
							if ((neighborsNeighborflags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE) {
								//take this one
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
								neighborHasNeighbor[splineCount]=true;
							} else {
								// use the corrected position
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
								neighborHasNeighbor[splineCount]=true;
							}
						}
							//investigate neighbors neighborhood and add control points if there are any
						if( (neighborflags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE) {
							//get neighbors neighbors flag
							int neighborsNeighborIndex = getNeighborIndex(neighbors.y, EAST, 0);
							int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
							if ((neighborsNeighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE) {
								//take this one
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
								neighborHasNeighbor[splineCount]=true;
							} else {
								// use the corrected position
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
								neighborHasNeighbor[splineCount]=true;
							}
						}
						if( (neighborflags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE) {
							//get neighbors neighbors flag
							int neighborsNeighborIndex = getNeighborIndex(neighbors.y, SOUTH, 0);
							int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
							if ((neighborsNeighborflags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE) {
								//take this one
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
								neighborHasNeighbor[splineCount]=true;
							} else {
								// use the corrected position
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
								neighborHasNeighbor[splineCount]=true;
							}
						}
					}
					splineCount++;
				} else if ( (neighborflags & HAS_CORRECTED_POSITION) == HAS_CORRECTED_POSITION) {
					linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.y+1).rg;
					linearCount++;
				} else {
					linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.y).rg;
					linearCount++;
				}
			}
			
			if( (flags & HAS_SOUTHERN_NEIGHBOR) == HAS_SOUTHERN_NEIGHBOR ) {
				int neighborflags = texelFetch(CPflags,neighbors.z).r;
				if( (flags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE ) {
					if(neighborflags >= HAS_NORTHERN_SPLINE && !( (neighborflags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE )) {
						splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.z+1).rg;
					} else {
						splineNeighbors[splineCount] = texelFetch(indexedCellPositions,neighbors.z).rg;
						//investigate neighbors neighborhood and add control points if there are any
						if( (neighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE) {
							//get neighbors neighbors flag
							int neighborsNeighborIndex = getNeighborIndex(neighbors.z, WEST, 0);
							int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
							if ((neighborsNeighborflags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE) {
								//take this one
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
								neighborHasNeighbor[splineCount]=true;
							} else {
								// use the corrected position
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
								neighborHasNeighbor[splineCount]=true;
							}
						}
							//investigate neighbors neighborhood and add control points if there are any
						if( (neighborflags & HAS_EASTERN_SPLINE) == HAS_EASTERN_SPLINE) {
							//get neighbors neighbors flag
							int neighborsNeighborIndex = getNeighborIndex(neighbors.z, EAST, 0);
							int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
							if ((neighborsNeighborflags & HAS_WESTERN_SPLINE) == HAS_WESTERN_SPLINE) {
								//take this one
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
								neighborHasNeighbor[splineCount]=true;
							} else {
								// use the corrected position
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
								neighborHasNeighbor[splineCount]=true;
							}
						}
						if( (neighborflags & HAS_SOUTHERN_SPLINE) == HAS_SOUTHERN_SPLINE) {
							//get neighbors neighbors flag
							int neighborsNeighborIndex = getNeighborIndex(neighbors.z, SOUTH, 0);
							int neighborsNeighborflags = texelFetch(CPflags,neighborsNeighborIndex).r;
							if ((neighborsNeighborflags & HAS_NORTHERN_SPLINE) == HAS_NORTHERN_SPLINE) {
								//take this one
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex).rg;
								neighborHasNeighbor[splineCount]=true;
							} else {
								// use the corrected position
								splineNeighborsNeighbors[splineCount] = texelFetch(indexedCellPositions,neighborsNeighborIndex+1).rg;
								neighborHasNeighbor[splineCount]=true;
							}
						}
					}
					splineCount++;
				} else if ( (neighborflags & HAS_CORRECTED_POSITION) == HAS_CORRECTED_POSITION) {
					linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.z+1).rg;
					linearCount++;
				} else {
					linearNeighbors[linearCount] = texelFetch(indexedCellPositions,neighbors.z).rg;
					linearCount++;
				}
			}
			
			//find intersections to determine pixel influence
			if(splineCount == 2) {
				if(neighborHasNeighbor[0]) {
					vec2 pA = calcSplinePoint(splineNeighborsNeighbors[0], splineNeighbors[0], knot1pos, 0.0);
					for (float t = STEP ; t <= 1.0; t = t + STEP) {
						vec2 pointB = calcSplinePoint(splineNeighborsNeighbors[0], splineNeighbors[0], knot1pos, t);
						//evaluate interections
						if( interects(cellSpaceCoords, ULCoords, pA, pointB) ) {
							influencingPixels.x = false;
						}
						if( interects(cellSpaceCoords, URCoords, pA, pointB) ) {
							influencingPixels.y = false;
						}
						if( interects(cellSpaceCoords, LLCoords, pA, pointB) ) {
							influencingPixels.z = false;
						}
						if( interects(cellSpaceCoords, LRCoords, pA, pointB) ) {
							influencingPixels.w = false;
						}
						pA = pointB;
					}
				}
				vec2 pointA = calcSplinePoint(splineNeighbors[0], knot1pos, splineNeighbors[1], 0.0);
				for (float t = STEP ; t <= 1.0 ; t = t + STEP) {
					vec2 pointB = calcSplinePoint(splineNeighbors[0], knot1pos, splineNeighbors[1], t);
					//evaluate interections
					if( interects(cellSpaceCoords, ULCoords, pointA, pointB) ) {
						influencingPixels.x = false;
					}
					if( interects(cellSpaceCoords, URCoords, pointA, pointB) ) {
						influencingPixels.y = false;
					}
					if( interects(cellSpaceCoords, LLCoords, pointA, pointB) ) {
						influencingPixels.z = false;
					}
					if( interects(cellSpaceCoords, LRCoords, pointA, pointB) ) {
						influencingPixels.w = false;
					}
					pointA = pointB;
				}
				if(neighborHasNeighbor[1]) {
					vec2 pA = calcSplinePoint(knot1pos, splineNeighbors[1], splineNeighborsNeighbors[1], 0.0);
					for (float t = STEP ; t <= 1.0; t = t + STEP) {
						vec2 pointB = calcSplinePoint(knot1pos, splineNeighbors[1],  splineNeighborsNeighbors[1], t);
						//evaluate interections
						if( interects(cellSpaceCoords, ULCoords, pA, pointB) ) {
							influencingPixels.x = false;
						}
						if( interects(cellSpaceCoords, URCoords, pA, pointB) ) {
							influencingPixels.y = false;
						}
						if( interects(cellSpaceCoords, LLCoords, pA, pointB) ) {
							influencingPixels.z = false;
						}
						if( interects(cellSpaceCoords, LRCoords, pA, pointB) ) {
							influencingPixels.w = false;
						}
						pA = pointB;
					}
				}
			}
			for(int i = 0 ; i < linearCount ; i++) {
				vec2 pointA = knot1pos;
				/*
				for (float t = STEP ; t <= 1.0; t = t + STEP + LINEAR_SEGMENT_LENGTH) {
					vec2 pointB = calcSplineEndPoint(knot1pos, linearNeighbors[i], t);
					//evaluate interections
					if( interects(cellSpaceCoords, ULCoords, pA, pointB) ) {
						influencingPixels.x = false;
					}
					if( interects(cellSpaceCoords, URCoords, pA, pointB) ) {
						influencingPixels.y = false;
					}
					if( interects(cellSpaceCoords, LLCoords, pA, pointB) ) {
						influencingPixels.z = false;
					}
					if( interects(cellSpaceCoords, LRCoords, pA, pointB) ) {
						influencingPixels.w = false;
					}
					pA = pointB;
				}*/
				vec2 pointB = mix(knot1pos, linearNeighbors[i], LINEAR_SEGMENT_LENGTH);
				if( interects(cellSpaceCoords, ULCoords, pointA, pointB) ) {
					influencingPixels.x = false;
				}
				if( interects(cellSpaceCoords, URCoords, pointA, pointB) ) {
					influencingPixels.y = false;
				}
				if( interects(cellSpaceCoords, LLCoords, pointA, pointB) ) {
					influencingPixels.z = false;
				}
				if( interects(cellSpaceCoords, LRCoords, pointA, pointB) ) {
					influencingPixels.w = false;
				}
			}
		}	
	}

	//int influencingPixelCount = 0;
	//float weights[16];
	vec4 colorSum = vec4(0.0,0.0,0.0,0.0);
	float weightSum = 0.0;
	//influencingPixels order: UL UR LL LR
	if(influencingPixels.x) {
		//calculate influence of the Pixel
		vec4 col = texelFetch(pixelArt, ivec2(ULCoords),0);
		float dist = distance(cellSpaceCoords,ULCoords);
		float weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
		colorSum += col * weight;
		weightSum += weight;
		//checkout this pixels connected neigbors
		ivec2 lookupcoords = ivec2(2 * ULCoords + 1);
		int edges = int(texelFetch(similarityGraph, lookupcoords ,0).g);
		//calculate weights for those pixels
		if( (edges & SOUTHWEST) == SOUTHWEST ) {
			col = texelFetch(pixelArt, ivec2(ULCoords.x-1,ULCoords.y-1),0);
			dist = distance(cellSpaceCoords,vec2(ULCoords.x-1,ULCoords.y-1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & WEST) == WEST ) {
			col = texelFetch(pixelArt, ivec2(ULCoords.x-1,ULCoords.y),0);
			dist = distance(cellSpaceCoords,vec2(ULCoords.x-1,ULCoords.y));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & NORTHWEST) == NORTHWEST ) {
			col = texelFetch(pixelArt, ivec2(ULCoords.x-1,ULCoords.y+1),0);
			dist = distance(cellSpaceCoords,vec2(ULCoords.x-1,ULCoords.y+1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & NORTH) == NORTH ) {
			col = texelFetch(pixelArt, ivec2(ULCoords.x,ULCoords.y+1),0);
			dist = distance(cellSpaceCoords,vec2(ULCoords.x,ULCoords.y+1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & NORTHEAST) == NORTHEAST ) {
			col = texelFetch(pixelArt, ivec2(ULCoords.x+1,ULCoords.y+1),0);
			dist = distance(cellSpaceCoords,vec2(ULCoords.x+1,ULCoords.y+1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		//colors[influencingPixelCount] = texelFetch(pixelArt, ivec2(ULCoords),0);
		//influencingPixelCount++;
	}
	if(influencingPixels.y) {
		//calculate influence of the Pixel
		vec4 col = texelFetch(pixelArt, ivec2(URCoords),0);
		float dist = distance(cellSpaceCoords,URCoords);
		float weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
		colorSum += col * weight;
		weightSum += weight;
		//checkout this pixels connected neigbors
		ivec2 lookupcoords = ivec2(2 * URCoords + 1);
		int edges = int(texelFetch(similarityGraph, lookupcoords ,0).g);
		//calculate weights for those pixels
		if( (edges & NORTHWEST) == NORTHWEST ) {
			col = texelFetch(pixelArt, ivec2(URCoords.x-1,URCoords.y+1),0);
			dist = distance(cellSpaceCoords,vec2(URCoords.x-1,URCoords.y+1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & NORTH) == NORTH ) {
			col = texelFetch(pixelArt, ivec2(URCoords.x,URCoords.y+1),0);
			dist = distance(cellSpaceCoords,vec2(URCoords.x,URCoords.y+1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & NORTHEAST) == NORTHEAST ) {
			col = texelFetch(pixelArt, ivec2(URCoords.x+1,URCoords.y+1),0);
			dist = distance(cellSpaceCoords,vec2(URCoords.x+1,URCoords.y+1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & EAST) == EAST ) {
			col = texelFetch(pixelArt, ivec2(URCoords.x+1,URCoords.y),0);
			dist = distance(cellSpaceCoords,vec2(URCoords.x+1,URCoords.y));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & SOUTHEAST) == SOUTHEAST ) {
			col = texelFetch(pixelArt, ivec2(URCoords.x+1,URCoords.y-1),0);
			dist = distance(cellSpaceCoords,vec2(URCoords.x+1,URCoords.y-1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		// colors[influencingPixelCount] = texelFetch(pixelArt, ivec2(URCoords),0);
		// influencingPixelCount++;
	}
	if(influencingPixels.z) {
		//calculate influence of the Pixel
		vec4 col = texelFetch(pixelArt, ivec2(LLCoords),0);
		float dist = distance(cellSpaceCoords,LLCoords);
		float weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
		colorSum += col * weight;
		weightSum += weight;
		//checkout this pixels connected neigbors
		ivec2 lookupcoords = ivec2(2 * LLCoords + 1);
		int edges = int(texelFetch(similarityGraph, lookupcoords ,0).g);
		//calculate weights for those pixels
		if( (edges & NORTHWEST) == NORTHWEST ) {
			col = texelFetch(pixelArt, ivec2(LLCoords.x-1,LLCoords.y+1),0);
			dist = distance(cellSpaceCoords,vec2(LLCoords.x-1,LLCoords.y+1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & WEST) == WEST ) {
			col = texelFetch(pixelArt, ivec2(LLCoords.x-1,LLCoords.y),0);
			dist = distance(cellSpaceCoords,vec2(LLCoords.x-1,LLCoords.y));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & SOUTHWEST) == SOUTHWEST ) {
			col = texelFetch(pixelArt, ivec2(LLCoords.x-1,LLCoords.y-1),0);
			dist = distance(cellSpaceCoords,vec2(LLCoords.x-1,LLCoords.y-1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & SOUTH) == SOUTH ) {
			col = texelFetch(pixelArt, ivec2(LLCoords.x,LLCoords.y-1),0);
			dist = distance(cellSpaceCoords,vec2(LLCoords.x,LLCoords.y-1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & SOUTHEAST) == SOUTHEAST ) {
			col = texelFetch(pixelArt, ivec2(LLCoords.x+1,LLCoords.y-1),0);
			dist = distance(cellSpaceCoords,vec2(LLCoords.x+1,LLCoords.y-1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		// colors[influencingPixelCount] = texelFetch(pixelArt, ivec2(LLCoords),0);
		// influencingPixelCount++;
	}
	if(influencingPixels.w) {
		//calculate influence of the Pixel
		vec4 col = texelFetch(pixelArt, ivec2(LRCoords),0);
		float dist = distance(cellSpaceCoords,LRCoords);
		float weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
		colorSum += col * weight;
		weightSum += weight;
		//checkout this pixels connected neigbors
		ivec2 lookupcoords = ivec2(2 * LRCoords + 1);
		int edges = int(texelFetch(similarityGraph, lookupcoords ,0).g);
		//calculate weights for those pixels
		if( (edges & NORTHEAST) == NORTHEAST ) {
			col = texelFetch(pixelArt, ivec2(LRCoords.x+1,LRCoords.y+1),0);
			dist = distance(cellSpaceCoords,vec2(LRCoords.x+1,LRCoords.y+1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & EAST) == EAST ) {
			col = texelFetch(pixelArt, ivec2(LRCoords.x+1,LRCoords.y),0);
			dist = distance(cellSpaceCoords,vec2(LRCoords.x+1,LRCoords.y));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & SOUTHWEST) == SOUTHWEST ) {
			col = texelFetch(pixelArt, ivec2(LRCoords.x-1,LRCoords.y-1),0);
			dist = distance(cellSpaceCoords,vec2(LRCoords.x-1,LRCoords.y-1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & SOUTH) == SOUTH ) {
			col = texelFetch(pixelArt, ivec2(LRCoords.x,LRCoords.y-1),0);
			dist = distance(cellSpaceCoords,vec2(LRCoords.x,LRCoords.y-1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		if( (edges & SOUTHEAST) == SOUTHEAST ) {
			col = texelFetch(pixelArt, ivec2(LRCoords.x+1,LRCoords.y-1),0);
			dist = distance(cellSpaceCoords,vec2(LRCoords.x+1,LRCoords.y-1));
			weight = exp(-(dist*dist)*GAUSS_MULTIPLIER);
			colorSum += col * weight;
			weightSum += weight;
		}
		// colors[influencingPixelCount] = texelFetch(pixelArt, ivec2(LRCoords),0);
		// influencingPixelCount++;
	}
	// vec4 mixedColor = vec4(0.0,0.0,0.0,0.0);
	// for(int i = 0 ; i < influencingPixelCount ; i++) {
		// mixedColor += colors[i];
	// }
	// if(influencingPixelCount == 0) {
		// influencingPixelCount = 1;
	// }
	// color0 = mixedColor/influencingPixelCount;
	// color0 = colorSum/weightSum;
	//debug only
	// if(any (influencingPixels) ) {
		color0 = colorSum/weightSum;
	// } else {
		// color0 = vec4(0.0,1.0,1.0,0.0);
	// }
	
}
